#include <msp430.h> 
#include <math.h>

volatile unsigned long StoredCount = 0; // HeNe fringe count
volatile unsigned long StoredCount2 = 0; // Unknown fringe count

volatile unsigned int start = 0, start2; // start/stop signals
volatile unsigned int stop = 0;

int i = 0;
const int n = 10;

double wavelength [n];
double std;
double avg;
double calculateSD(double data[]);
double calculatemean(double data[]);


int main(void)
{
  WDTCTL = WDTPW + WDTHOLD;	// stop watchdog timer
  
  FRCTL0 = FRCTLPW|NWAITS_1;
  CSCTL0_H = CSKEY >> 8;            // Unlock CS registers
  CSCTL1 = DCORSEL|DCOFSEL_4;       // Set DCORSEL = 1 and DCOFSEL = 4, this sets the DCO to 16MHz (pg. 104 of manual A,pg. 44 of manual B)
  CSCTL2 |= SELS__DCOCLK;           // Set the DCO clock as the SMCLK source (pg.105 manual A)
  CSCTL3 = DIVS_0;                  // divide by 1 (pg. 106 manual A)
  CSCTL0_H = 0x00;                  // Lock CS module (use byte mode to upper byte)

  __delay_cycles(1000);                     // Allow clock system to settle

	
 // set up the input pins for capture compare interrupt functions on on Timer A
 // pin 1.3 is timer A1.2
 // pin 1.5 is timer A0.0
 // pin 1.6 is timer A0.1
 // pin 1.7 is timer A0.2 (I don't think the final working version of the code actually uses pin 1.7 for anything, but I left in here just in case)
 
 
  
  // configure pin 1.6 for TA0.CCI1A function
  P1DIR &= ~BIT6; //set P1DIR.6 = 0
  P1SEL1 |= BIT6; //set P1SEL1.6 = 1
  P1SEL0 |= BIT6; //set P1SEL0.6 = 1
  
  // configure pin 1.7 for TA0.CCI2A function
  P1DIR &= ~BIT7; //set P1DIR.7 = 0
  P1SEL1 |= BIT7; //set P1SEL1.7 = 1
  P1SEL0 |= BIT7; //set P1SEL0.7 = 1
  
 // configure pin 1.3 for TA1.CCI2A function
  P1DIR &= ~BIT3;
  P1SEL1 &= ~BIT3;
  P1SEL0 |= BIT3;
  
  //configure pin 1.5 for TA0.CCI0A function
  P1DIR &= ~BIT5;
  P1SEL1 |= BIT5;
  P1SEL0 |= BIT5;
 
  
  
  

  PM5CTL0 &= ~LOCKLPM5; // not sure what this does, it was just here in the template when I started writing the code
  
  
  //configure interrupt settings on each capture control register, info about the TACCTL registers is on pg. 657 of manual A
  
  TA1CCTL0 |= CCIE + CCIS_0 + CM_1 + CAP; // I don't think this line is needed anymore but i just left it, it's from when I was using pin 1.4 which isn't being used in the code anymore 
                                          
  TA1CCTL2 |= CCIE + CCIS_0 + CM_1 + CAP;   // capture interrupt enable + capture off of CCI0A + capture on rising edge + turn on capture mode
                                            
                                            
  TA0CCTL0 |= CCIE + CCIS_0 + CM_1 + CAP;   // same
  
  TA0CCTL1 |= CCIE + CCIS_0 + CM_2 + CAP;   // same except CM_2 sets it to capture on falling edge
  
  TA0CCTL2 |= CCIE + CCIS_0 + CM_2 + CAP;   // this line might not be needed, TA0CCTL2 is for capture control registers on pin 1.7 which isn't being used 
  
  
  
                                          
  //set timer A clock source as SMCLK (pg. 655 manual A)
  
  TA1CTL |= TASSEL_2; // source timer A1 from SMCLK
  TA0CTL |= TASSEL_2; // source timer A0 from SMCLK
  
  
  _BIS_SR(GIE); // i think this enables interrupts, and can be undone with the __disable_interrupt() command found in the while loop below


  	while(1)
	{
	    //TA1CTL |= MC1;		// Start timer
	    while(start == 0);
	    start2=0;
	    while(start2 == 0); // can't remember why I made the start signal this way the start and start2 variables but it works 
	    
	    __disable_interrupt(); // turn off interrupt while zeroing the fringe counts in case there is any time delay in zeroing the variables
	    StoredCount = 0; // zero the fringe counts
        StoredCount2 = 0;
         _BIS_SR(GIE); // turn on interrupts
	    while(StoredCount < 130000); // count until specified amount in StoredCount
    	wavelength[i] = (StoredCount*632.8)/StoredCount2; // calculate wavelength
	    StoredCount = 0; // rezero the counts, this probably isn't necessary because they are zeroed before the next counting period anyways
        StoredCount2 = 0;
        start = 0;          //rezero start variable
	    TA1R = 0;           // Zero Timer_A1 register
	    
	    // calculate mean and standard deviation of wavelength measurements
	    // if you wait long enough and the # of measurements goes over number specified it will just start writing over the measurements starting with the first one
	    i++;            
	    if (i == n){
	        i=0;
	        avg = calculatemean(wavelength);
	        std = calculateSD(wavelength);
	    }
	}
	return 0;
}


double calculatemean(double data[])
{
    double sum = 0.0, mean;
    int j;
    for(j=0; j<n; ++j)
    {
        sum += data[j];
    }
    mean = sum/n;
    return mean;
}


double calculateSD(double data[])
{
    double sum = 0.0, mean, standardDeviation = 0.0;
    int j;
    for(j=0; j<n; ++j)
    {
        sum += data[j];
    }
    mean = sum/n;
    for(j=0; j<n; ++j)
        standardDeviation += pow(data[j] - mean, 2);
    return sqrt(standardDeviation/(n-1));

}




// these are the operations that are done when an interrupt is triggered

// i could never figure out the naming system for the interrupt functions (such as TIMER0_A0_VECTOR  below ) but it definetly 
// matters and the code won't work if they aren't named correctly


//interrupt for HeNe fringe
#pragma vector = TIMER0_A0_VECTOR
__interrupt void Timer0_A0_ISR (void)
{
    switch(TA0IV){
    case 0x00:
        StoredCount++; // +1 for Hene 
        start2++;      // not sure why I added 1 to the start2 variable each time Hene fringe was detected, maybe try it without this
 
    break;}
}



#pragma vector = TIMER0_A1_VECTOR
__interrupt void Timer0_A1_ISR (void)
{
    switch(TA0IV){
    case 0x02:
        start++; // +1 for start 
        break;
    case 0x04:
        stop++; // don't think the stop variable is actually used for anything so maybe try the code without this

    break;}
}




#pragma vector = TIMER1_A1_VECTOR
__interrupt void Timer1_A1_ISR (void)
{
    switch(TA1IV){
        case 0x04:
            StoredCount2++; // +1 unknown fringes
           
                break;
    }
}
    



